defmodule Akkoma2org.ActivityStream do
  @moduledoc """
  The ActivityStream context.
  """

  import Ecto.Query, warn: false
  alias Akkoma2org.Repo

  alias Akkoma2org.ActivityStream.Action

  @doc """
  Returns the list of actions.

  ## Examples

      iex> list_actions()
      [%Action{}, ...]

  """
  def list_actions do
    Repo.all(Action)
  end

  @doc """
  Gets a single action.

  Raises `Ecto.NoResultsError` if the Action does not exist.

  ## Examples

      iex> get_action!(123)
      %Action{}

      iex> get_action!(456)
      ** (Ecto.NoResultsError)

  """
  def get_action!(id), do: Repo.get!(Action, id)

  @doc """
  Creates a action.

  ## Examples

      iex> create_action(%{field: value})
      {:ok, %Action{}}

      iex> create_action(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_action(attrs \\ %{}) do
    %Action{}
    |> Action.changeset(attrs)
    |> Repo.insert()
  end

  @doc """
  Updates a action.

  ## Examples

      iex> update_action(action, %{field: new_value})
      {:ok, %Action{}}

      iex> update_action(action, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_action(%Action{} = action, attrs) do
    action
    |> Action.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes a action.

  ## Examples

      iex> delete_action(action)
      {:ok, %Action{}}

      iex> delete_action(action)
      {:error, %Ecto.Changeset{}}

  """
  def delete_action(%Action{} = action) do
    Repo.delete(action)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking action changes.

  ## Examples

      iex> change_action(action)
      %Ecto.Changeset{data: %Action{}}

  """
  def change_action(%Action{} = action, attrs \\ %{}) do
    Action.changeset(action, attrs)
  end

  def latest_action(), do: latest_actions(1) |> Enum.at(0)

  @doc """
  Returns the most-recent Action based on the publish date
  """
  def latest_actions(limit) do
    query =
      from a in Akkoma2org.ActivityStream.Action,
        order_by: [desc: a.published],
        limit: ^limit

    Repo.all(query)
  end

  @doc """
  Searchs our actions based on a simple query.

  ## Examples

  iex> search_actions("hello")
  [%Action{}, ...]

  """
  def search_actions(q) do
    from(d in Action,
      where: fragment("actions MATCH ?", ^q),
      order_by: [asc: :rank]
    )
    |> Repo.all()
  end

  def delete_dupes() do
    l = list_actions |> Enum.map(&Map.get(&1, :external_id))

    Enum.uniq(l -- Enum.uniq(l))
    |> Enum.map(
      &Akkoma2org.Repo.all(
        from a in Akkoma2org.ActivityStream.Action, where: a.external_id == ^&1
      )
    )
    |> Enum.map(&(its = Enum.slice(&1, 1..-1)))
    |> List.flatten()
    |> Enum.map(&Akkoma2org.ActivityStream.delete_action(&1))
  end
end
