defmodule Akkoma2org.ActivityStream.APIBackfill do
  alias Akkoma2org.ActivityStream
  alias Akkoma2org.ActivityStream.Action

  # https://stackoverflow.com/questions/47354778/elixir-map-with-mixed-keys
  def key_to_atom(map) do
    Enum.reduce(map, %{}, fn
      {key, value}, acc when is_atom(key) -> Map.put(acc, key, value)
      # String.to_existing_atom saves us from overloading the VM by
      # creating too many atoms. It'll always succeed because all the fields
      # in the database already exist as atoms at runtime.
      # lol haha yes
      {key, value}, acc when is_binary(key) -> Map.put(acc, String.to_atom(key), value)
    end)
  end

  def munge_mastodon(%Hunter.Status{reblog: status} = _orig_status) when is_map(status) do
    # nested Status has these come in as dicts... same w/ akkoma+pleroma extensions i made hm.
    acct_w_atoms =
      status.account
      |> key_to_atom()

    acct = Map.merge(%Hunter.Account{}, acct_w_atoms)

    %{status | account: acct} |> munge_mastodon()
  end

  def munge_mastodon(%Hunter.Status{reblog: nil} = status) do
    {:ok, full_action} = Poison.encode(status)

    post_source =
      case status.akkoma["source"] do
        str when is_binary(str) ->
          str

        %{"content" => content} ->
          content

        _ ->
          ""
      end

    # url
    %{
      external_id: status.id,
      # || status.account[:url],
      actor: status.account.url,
      content: status.content,
      post_source: post_source,
      dest: status.pleroma["context"],
      summary: status.spoiler_text,
      published: status.created_at,
      full_action: full_action
    }
  end

  def unfold(conn, account_id, post_id, :backward),
    do: unfold1(conn, account_id, post_id, :max_id)

  def unfold(conn, account_id, post_id, :forward),
    do: unfold1(conn, account_id, post_id, :since_id)

  defp unfold1(conn, account_id, last_id, dir_key) do
    # iterate over statuses
    # accumulator is next_id + collected statuses in a proplist
    Stream.unfold(
      {last_id, 0},
      fn
        # # early exit
        # {_, prev_count} when prev_count > 100 ->
        #   nil

        {last_id, prev_count} ->
          statuses = Hunter.statuses(conn, account_id, [{dir_key, last_id}])

          case statuses do
            [] ->
              nil

            _ ->
              last_id = statuses |> Enum.at(-1) |> Map.get(:id)
              # IO.puts(:stdio, all |> Enum.map(&Map.get(&1, :id)) |> Enum.join(" "))

              count = Enum.count(statuses) + prev_count

              IO.puts("Fetched #{count} now... up to #{last_id}")

              {statuses, {last_id, count}}
          end
      end
    )
  end

  def fetch_latest(user: user, password: pass) do
    {_app, conn, my_id} = initialize(user, pass)

    %Action{external_id: last_id} = ActivityStream.latest_action()

    unfold(conn, my_id, last_id, :forward)
    |> Enum.to_list()
    |> List.flatten()
    |> Enum.map(fn it ->
      case it
           |> munge_mastodon()
           |> Akkoma2org.ActivityStream.create_action() do
        {:ok, cs} -> cs
        {:error, cs} -> IO.inspect(cs)
      end
    end)
  end

  def backfill(user: user, password: pass) do
    {_app, conn, my_id} = initialize(user, pass)

    statuses = Hunter.statuses(conn, my_id)
    last_id = statuses |> Enum.at(-1) |> Map.get(:id)

    posts =
      unfold(conn, my_id, last_id, :backward)
      |> Enum.to_list()

    all_posts = (statuses ++ posts) |> List.flatten()

    all_posts
    |> Enum.map(fn it ->
      case it
           |> munge_mastodon()
           |> Akkoma2org.ActivityStream.create_action() do
        {:ok, cs} -> cs
        {:error, cs} -> IO.inspect(cs)
      end
    end)
  end

  defp initialize(user, pass) do
    app =
      try do
        Hunter.Application.load_credentials("akkoma2org")
      rescue
        FileError ->
          Hunter.create_app("akkoma2org", "urn:ietf:wg:oauth:2.0:oob", ["read"], nil, save?: true)
      end

    conn = Hunter.log_in(app, user, pass, "https://notes.whatthefuck.computer")

    my_id = Hunter.verify_credentials(conn) |> Map.get(:id)

    {app, conn, my_id}
  end
end
