defmodule Akkoma2org.ActivityStreamTest do
  use Akkoma2org.DataCase

  alias Akkoma2org.ActivityStream

  describe "actions" do
    alias Akkoma2org.ActivityStream.Action

    import Akkoma2org.ActivityStreamFixtures

    @invalid_attrs %{
      action_type: nil,
      actor: nil,
      content: nil,
      dest: nil,
      external_id: nil,
      published: nil,
      post_source: nil,
      summary: nil,
      type: nil
    }

    test "list_actions/0 returns all actions" do
      action = action_fixture()
      assert ActivityStream.list_actions() == [action]
    end

    test "get_action!/1 returns the action with given id" do
      action = action_fixture()
      assert ActivityStream.get_action!(action.id) == action
    end

    test "create_action/1 with valid data creates a action" do
      valid_attrs = %{
        action_type: "some action_type",
        actor: "some actor",
        content: "some content",
        dest: "some dest",
        external_id: "some external_id",
        published: "some published",
        post_source: "some source",
        summary: "some summary",
        type: "some type"
      }

      assert {:ok, %Action{} = action} = ActivityStream.create_action(valid_attrs)
      assert action.action_type == "some action_type"
      assert action.actor == "some actor"
      assert action.content == "some content"
      assert action.dest == "some dest"
      assert action.external_id == "some external_id"
      assert action.published == "some published"
      assert action.post_source == "some source"
      assert action.summary == "some summary"
      assert action.type == "some type"
    end

    test "create_action/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = ActivityStream.create_action(@invalid_attrs)
    end

    test "update_action/2 with valid data updates the action" do
      action = action_fixture()

      update_attrs = %{
        action_type: "some updated action_type",
        actor: "some updated actor",
        content: "some updated content",
        dest: "some updated dest",
        external_id: "some updated external_id",
        published: "some updated published",
        post_source: "some updated source",
        summary: "some updated summary",
        type: "some updated type"
      }

      assert {:ok, %Action{} = action} = ActivityStream.update_action(action, update_attrs)
      assert action.action_type == "some updated action_type"
      assert action.actor == "some updated actor"
      assert action.content == "some updated content"
      assert action.dest == "some updated dest"
      assert action.external_id == "some updated external_id"
      assert action.published == "some updated published"
      assert action.post_source == "some updated source"
      assert action.summary == "some updated summary"
      assert action.type == "some updated type"
    end

    test "update_action/2 with invalid data returns error changeset" do
      action = action_fixture()
      assert {:error, %Ecto.Changeset{}} = ActivityStream.update_action(action, @invalid_attrs)
      assert action == ActivityStream.get_action!(action.id)
    end

    test "delete_action/1 deletes the action" do
      action = action_fixture()
      assert {:ok, %Action{}} = ActivityStream.delete_action(action)
      assert_raise Ecto.NoResultsError, fn -> ActivityStream.get_action!(action.id) end
    end

    test "change_action/1 returns a action changeset" do
      action = action_fixture()
      assert %Ecto.Changeset{} = ActivityStream.change_action(action)
    end
  end

  describe "actions" do
    alias Akkoma2org.ActivityStream.Action

    import Akkoma2org.ActivityStreamFixtures

    @invalid_attrs %{action_type: nil, actor: nil, content: nil, dest: nil, external_id: nil, full_action: nil, post_source: nil, published: nil, rank: nil, summary: nil, type: nil}

    test "list_actions/0 returns all actions" do
      action = action_fixture()
      assert ActivityStream.list_actions() == [action]
    end

    test "get_action!/1 returns the action with given id" do
      action = action_fixture()
      assert ActivityStream.get_action!(action.id) == action
    end

    test "create_action/1 with valid data creates a action" do
      valid_attrs = %{action_type: "some action_type", actor: "some actor", content: "some content", dest: "some dest", external_id: "some external_id", full_action: "some full_action", post_source: "some post_source", published: "some published", rank: 120.5, summary: "some summary", type: "some type"}

      assert {:ok, %Action{} = action} = ActivityStream.create_action(valid_attrs)
      assert action.action_type == "some action_type"
      assert action.actor == "some actor"
      assert action.content == "some content"
      assert action.dest == "some dest"
      assert action.external_id == "some external_id"
      assert action.full_action == "some full_action"
      assert action.post_source == "some post_source"
      assert action.published == "some published"
      assert action.rank == 120.5
      assert action.summary == "some summary"
      assert action.type == "some type"
    end

    test "create_action/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = ActivityStream.create_action(@invalid_attrs)
    end

    test "update_action/2 with valid data updates the action" do
      action = action_fixture()
      update_attrs = %{action_type: "some updated action_type", actor: "some updated actor", content: "some updated content", dest: "some updated dest", external_id: "some updated external_id", full_action: "some updated full_action", post_source: "some updated post_source", published: "some updated published", rank: 456.7, summary: "some updated summary", type: "some updated type"}

      assert {:ok, %Action{} = action} = ActivityStream.update_action(action, update_attrs)
      assert action.action_type == "some updated action_type"
      assert action.actor == "some updated actor"
      assert action.content == "some updated content"
      assert action.dest == "some updated dest"
      assert action.external_id == "some updated external_id"
      assert action.full_action == "some updated full_action"
      assert action.post_source == "some updated post_source"
      assert action.published == "some updated published"
      assert action.rank == 456.7
      assert action.summary == "some updated summary"
      assert action.type == "some updated type"
    end

    test "update_action/2 with invalid data returns error changeset" do
      action = action_fixture()
      assert {:error, %Ecto.Changeset{}} = ActivityStream.update_action(action, @invalid_attrs)
      assert action == ActivityStream.get_action!(action.id)
    end

    test "delete_action/1 deletes the action" do
      action = action_fixture()
      assert {:ok, %Action{}} = ActivityStream.delete_action(action)
      assert_raise Ecto.NoResultsError, fn -> ActivityStream.get_action!(action.id) end
    end

    test "change_action/1 returns a action changeset" do
      action = action_fixture()
      assert %Ecto.Changeset{} = ActivityStream.change_action(action)
    end
  end
end
